using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;

namespace CodeFirstMVC3.Domain.DataAccess
{
    public interface IRepository<T> where T : class
    {
        /// <summary>
        /// Create New Instance
        /// </summary>
        /// <returns></returns>
        T NewEntity();

        /// <summary>
        /// Inserts an item
        /// </summary>
        /// <param name="item"></param>
        void Add(T item);

        /// <summary>
        /// Deletes an item
        /// </summary>
        /// <param name="item"></param>
        void Remove(T item);

        /// <summary>
        /// Get an item matching to prediate
        /// </summary>
        /// <param name="predicate"></param>
        /// <returns></returns>
        T Get(Func<T, bool> predicate);

        /// <summary>
        /// Get Count of items matching to predicate
        /// </summary>
        /// <param name="predicate"></param>
        /// <returns></returns>
        int GetCount(Func<T, bool> predicate);

        /// <summary>
        /// Get IEnumerable object matching to predicate
        /// </summary>
        /// <param name="predicate"></param>
        /// <returns></returns>
        IEnumerable<T> GetAll(Func<T, bool> predicate);

        /// <summary>
        /// Get IEnumerable object matching to predicate,start index and count
        /// </summary>
        /// <param name="predicate"></param>
        /// <param name="start"></param>
        /// <param name="count"></param>
        /// <returns></returns>
        IEnumerable<T> GetAll(Func<T, bool> predicate, int start, int count);

        IEnumerable<K> GetQuery<K>(string queryString, Func<K, bool> predicate = null) where K : class;

        /// <summary>
        /// Saves the pending changes back into the DataContext.
        /// </summary>
        void Save();
    }
}